@@ -1,6 +1,6 @@
 package Data::Verifier;
 BEGIN {
-  $Data::Verifier::VERSION = '0.45';
+  $Data::Verifier::VERSION = '0.46';
 }
 use Moose;
 
@@ -40,20 +40,21 @@ sub coercion {
 }
 
 sub verify {
-    my ($self, $params) = @_;
+    my ($self, $params, $members) = @_;
 
     my $results = Data::Verifier::Results->new;
     my $profile = $self->profile;
 
     my $blessed_params = blessed($params);
 
+    my $skip_string_checks = 0;
     my @post_checks = ();
     foreach my $key (keys(%{ $profile })) {
 
         # Get the profile part that is pertinent to this field
         my $fprof = $profile->{$key};
 
-
+        # Deal with the fact that what we're given may be an object.
         my $val = do {
             if($blessed_params) {
                 $params->can($key) ? $params->$key() : undef;
@@ -62,75 +63,145 @@ sub verify {
             }
         };
 
-        my $oval = $val;
-
+        # Creat the "field" that we'll put into the result.
         my $field = Data::Verifier::Field->new;
+
+        # Save the original value.
         if(ref($val) eq 'ARRAY') {
-            my @values = @{ $val };
+            my @values = @{ $val }; # Make a copy of the array
             $field->original_value(\@values);
         } else {
             $field->original_value($val);
         }
 
-        # Pass through global filters
-        if($self->filters && defined $val) {
-            $val = $self->_filter_value($self->filters, $val);
-        }
-
-        # And now per-field ones
-        if($fprof->{filters} && defined $val) {
-            $val = $self->_filter_value($fprof->{filters}, $val);
-        }
-
-        # Empty strings are undefined
-        if(defined($val) && $val eq '') {
-            $val = undef;
-        }
-
-        if(ref($val)) {
-            my @values = @{ $val };
-            $field->post_filter_value(\@values);
-        } else {
-            $field->post_filter_value($val);
-        }
-
-        if($fprof->{required} && !defined($val)) {
-            # Set required fields to undef, as they are missing
-            $results->set_field($key, undef);
-        } else {
-            $results->set_field($key, $field);
-        }
-
-        # No sense in continuing if the value isn't defined.
-        next unless defined($val);
-
-        # Check min length
-        if($fprof->{min_length} && length($val) < $fprof->{min_length}) {
-            $field->reason('min_length');
-            $field->valid(0);
-            next; # stop processing!
-        }
-
-        # Check max length
-        if($fprof->{max_length} && length($val) > $fprof->{max_length}) {
-            $field->reason('max_length');
-            $field->valid(0);
-            next; # stop processing!
-        }
-
-        # Validate it
-        if(defined($val) && $fprof->{type}) {
+        # Early type check to catch parameterized ArrayRefs.
+        if($fprof->{type}) {
             my $cons = Moose::Util::TypeConstraints::find_or_parse_type_constraint($fprof->{type});
 
             die "Unknown type constraint '$fprof->{type}'" unless defined($cons);
 
+            # If this type is a paramterized arrayref, then we'll handle each
+            # param as if it was an independet value and run it through the
+            # whole profile.
+            if($cons->is_a_type_of('ArrayRef') && $cons->can('type_parameter')) {
+
+                # Get the type parameter for this arrayref
+                my $tc = $cons->type_parameter;
+                # Copy the profile.
+                my %prof_copy = %{ $fprof };
+                $prof_copy{type} = $tc->name;
+                my $dv = Data::Verifier->new(
+                    # Use the global filters
+                    filters => $self->filters,
+                    # And JUST this field
+                    profile => { $key => \%prof_copy }
+                );
+
+                # Make sure we are dealing with an array
+                my @possibles;
+                if(ref($val) eq 'ARRAY') {
+                    @possibles = @{ $val };
+                } else {
+                    @possibles = ( $val );
+                }
+
+                # So we can keep up with passed values.
+                my @passed;
+                my @pass_post_filter;
+                # Verify each one
+                foreach my $poss (@possibles) {
+                    my $res = $dv->verify({ $key => $poss }, 1);
+                    if($res->success) {
+                        # We need to keep up with passed values as well as
+                        # post filter values, copying them out of the result
+                        # for use in our field.
+                        push(@passed, $res->get_value($key));
+                        push(@pass_post_filter, $res->get_post_filter_value($key));
+                    } else {
+                        # Mark the whole field as failed.  We'll use this
+                        # later.
+                        $field->valid(0);
+                    }
+                }
+
+                # Set the value and post_filter_value for the field, then
+                # set the field in the result.  We're done, since we sorta
+                # recursed to check all the params for this field.
+                $val = \@passed;
+                $field->value(\@passed);
+                $field->post_filter_value(\@pass_post_filter);
+                $results->set_field($key, $field);
+
+                # Skip all the "string" checks, since we've already done all the
+                # real work.
+                $skip_string_checks = 1;
+            }
+            next unless $field->valid; # stop processing if invalid
+        }
+
+        unless($skip_string_checks) {
+            # Pass through global filters
+            if($self->filters && defined $val) {
+                $val = $self->_filter_value($self->filters, $val);
+            }
+
+            # And now per-field ones
+            if($fprof->{filters} && defined $val) {
+                $val = $self->_filter_value($fprof->{filters}, $val);
+            }
+
+            # Empty strings are undefined
+            if(defined($val) && $val eq '') {
+                $val = undef;
+            }
+
+            if(ref($val) eq 'ARRAY' ) {
+                my @values = @{ $val };
+                $field->post_filter_value(\@values);
+            } else {
+                $field->post_filter_value($val);
+            }
+
+            if($fprof->{required} && !defined($val)) {
+                # Set required fields to undef, as they are missing
+                $results->set_field($key, undef);
+            } else {
+                $results->set_field($key, $field);
+            }
+
+            # No sense in continuing if the value isn't defined.
+            next unless defined($val);
+
+            # Check min length
+            if($fprof->{min_length} && length($val) < $fprof->{min_length}) {
+                $field->reason('min_length');
+                $field->valid(0);
+                next; # stop processing!
+            }
+
+            # Check max length
+            if($fprof->{max_length} && length($val) > $fprof->{max_length}) {
+                $field->reason('max_length');
+                $field->valid(0);
+                next; # stop processing!
+            }
+        }
+
+        # Validate it
+        if($fprof->{type}) {
+            my $cons = Moose::Util::TypeConstraints::find_or_parse_type_constraint($fprof->{type});
+
+            die "Unknown type constraint '$fprof->{type}'" unless defined($cons);
+
+            # Look for a global coercion
             if($fprof->{coerce}) {
                 $val = $cons->coerce($val);
             }
+            # Try a one-off coercion.
             elsif(my $coercion = $fprof->{coercion}) {
                 $val = $coercion->coerce($val);
             }
-
+    
             unless($cons->check($val)) {
                 $field->reason('type_constraint');
                 $field->valid(0);
@@ -163,7 +234,12 @@ sub verify {
         }
 
         # Add this key the post check so we know to run through them
-        if(defined($fprof->{post_check}) && $fprof->{post_check}) {
+        if(!$members && defined($fprof->{post_check}) && $fprof->{post_check}) {
+            push(@post_checks, $key);
+        }
+        # Add this key to the post check if we're on "member" mode and there
+        # is a member_post_check specified
+        if($members && defined($fprof->{member_post_check}) && $fprof->{member_post_check}) {
             push(@post_checks, $key);
         }
 
@@ -179,7 +255,10 @@ sub verify {
             my $field = $results->get_field($key);
 
             # Execute the post_check...
-            my $pc = $fprof->{post_check};
+            
+            # If we are in member mode, use the member post check, else use
+            # plain ol' post check.
+            my $pc = $members ? $fprof->{member_post_check} : $fprof->{post_check};
             if(defined($pc) && $pc) {
                 try {
                     unless($results->$pc()) {
@@ -207,7 +286,9 @@ sub _filter_value {
     if(ref($filters) ne 'ARRAY') {
         $filters = [ $filters ];
     }
-    if(!ref($values)) {
+    # If we already have an array, just let it be. Otherwise transform the
+    # value into an array. ($values may also be a HashRef[Str] here)
+    unless ( ref $values eq 'ARRAY' ) {
         $created_ref = 1;
         $values = [ $values ];
     }
@@ -242,7 +323,7 @@ Data::Verifier - Profile based data verification with Moose type constraints.
 
 =head1 VERSION
 
-version 0.45
+version 0.46
 
 =head1 SYNOPSIS
 
@@ -303,6 +384,56 @@ It should be noted that if you choose to make a param a C<Str> then validation
 will fail if multiple values are provided.  To allow multiple values you
 must use an C<ArrayRef[Str]>.
 
+=head2 ArrayRef based types (more on Multiple Values)
+
+If you use an ArrayRef-based parameterized type (e.g. ArrayRef[Str]) then
+Data::Verifier has the following behavior:
+
+Each parameter supplied for the field is checked.  If all the members pass
+then the field is considered valid.  If any of the members fail, then the
+entire field is invalid.  If any of the members pass then those members will
+be included in the C<values> attribute.  An example:
+
+    use Moose::Util::TypeConstraints;
+    use Data::Verifier;
+
+    subtype 'Over10'
+    => as 'Num'
+    => where { $_ > 10 };
+
+    my $verifier = Data::Verifier->new(
+    profile => {
+        foos => {
+            type => 'ArrayRef[NumberOver10]',
+        }
+    }
+    );
+
+    my $res = $verifier->verify(foos => [ 1, 2, 30, 40 ]);
+    $res->success; # This is false, as 1 and 2 did not pass
+    $res->get_value('foos'); # [ 30, 40 ] because 30 and 40 passed!
+    $res->original_value('foos); # [ 1, 2, 30, 40 ] because it's all of them!
+
+It should also be noted that C<post_check>s that are specified in the profile
+do B<not> get applied to the individual members, only to the entire, completed
+field that they are constituents of.
+
+B<Note>: Filters and such DO get applied to individual fields, so something
+like:
+
+    my $verifier = Data::Verifier->new(
+      filters => qw(trim),
+      profile => {
+          foos => {
+              type => 'ArrayRef[Str]',
+              filters => 'collapse'
+          }
+      }
+    );
+
+In the above example, both C<trim> and C<collapse> B<bill> be applied to each
+member of foos.
+
 =head2 Stops on First Failure
 
 Data::Verifier stops checking a field (not all, just the failed one) if it
@@ -431,6 +562,14 @@ An optional length which the value may not exceed.
 
 An optional length which the value may not be less.
 
+=item B<member_post_check>
+
+A post check that is only to be applied to the members of an ArrayRef based
+type.  Because it is verified in something of a vacuum, the results object it
+receives will have no other values to look at.  Therefore member_post_check
+is only useful if you want to do some sort of weird post-check thing that I
+can't imagine would be a good idea.
+
 =item B<post_check>
 
 The C<post_check> key takes a subref and, after all verification has finished,
@@ -474,6 +613,10 @@ B<Note about post_check and exceptions>: If have a more complex post_check
 that could fail in multiple ways, you can C<die> in your post_check coderef
 and the exception will be stored in the fields C<reason> attribute.
 
+B<Note about post_check and ArrayRef based types>: The post check is B<not>
+executed for ArrayRef based types.  See the note earlier in this documentation
+about ArrayRefs.
+
 =item B<required>
 
 Determines if this field is required for verification.
